
#ifndef CAFFEINE_CAFMATH
#define CAFFEINE_CAFMATH

#include "default.h"

const float PI = 3.141592653589793f;
const float PIOVER180 = PI/180.0f;

const float DEGTORAD = PIOVER180;
const float RADTODEG = 1/PIOVER180;

const float EPSILON = 1e-6f;
const float INFINITY = 1e19f;

const float GRAVITY = -32.174f;

const double MAX_RAND_DOUBLE = (double)RAND_MAX;
                        
#define min(a,b) ((a < b) ? (a) : (b))
#define max(a,b) ((a > b) ? (a) : (b))

inline float absval(float a)
{
	return (a > 0) ? (a) : (-a);
}

inline int absval(int a)
{
	return (a > 0) ? (a) : (-a);
}

extern int RANDOM_SEED;

inline void randomize(float seed)
{
	RANDOM_SEED = (int)seed*1000;
}

inline float randfloat()
{
	return 0.00003051851f*(((RANDOM_SEED = RANDOM_SEED * 214013L + 2531011L) >> 16) & 0x7fff);
}

inline float sqrt(float x)
{
	int iter = 0;
	static float error = 0.0001f;
	float guess = x;
	float result = x;

	while(absval(x-guess*guess) > error)
	{
		guess = result;
		result = 0.5f*(guess+x/guess);
		if(++iter > 100)
			break;
	}

	return result;
}
/*
//modified Taylor series sine approximation
inline float sinf(float angle)
{
	float sq = angle*angle;
	float cu = sq*angle;
	float pe = cu*sq;
	float se = pe*sq;
	float no = se*sq;
	float el = no*sq;

	return angle-
		   0.166666666666f*cu+
		   0.008333333333f*pe-
		   0.000198412698f*se+
		   0.000002755732f*no-
		   0.000000025052f*el;
}

//modified Taylor series cosine approximation
inline float cosf(float angle)
{
	float sq = angle*angle;
	float qu = sq*sq;
	float he = qu*sq;
	float oc = he*sq;
	float de = oc*sq;

	return 1-
		   0.500000000000f*sq+
		   0.041666666666f*qu-
		   0.001388888888f*he+
		   0.000024801587f*oc-
		   0.000000275573f*de;
}

inline float tanf(float angle)
{
	return sinf(angle)/cosf(angle);
}

inline float atanf(float angle)
{
	float sq = angle*angle;
	float cu = sq*angle;
	float pe = cu*angle;
	float se = pe*angle;
	float no = se*angle;
	float el = no*angle;

	return angle - 
		   0.333333333333f*cu+
		   0.200000000000f*pe-
		   0.142857142857f*se+
		   0.111111111111f*no-
		   0.142857142857f*el;
}

float atan2f(float y, float x);
*/

#include <math.h>

#endif