
#include "cutscene1.h"

CCutScene1 *CCutScene1::Instance = 0;

void CCutScene1::Load()
{
	Engine = CEngine::Get();

	SceneLength = 25.4f;

	LogoWidth = 300.0f/2.0f;
	LogoHeight = 300.0f/2.0f;
	NumTextures = 4;
	LogoTime = SceneLength/(float)NumTextures;

	FlashTime = 0.0f;

	TextureList = new CTexture[NumTextures];
	TextureList[0].Load("data/cut1/zealous.bmp");
	TextureList[1].Load("data/cut1/caffeine.bmp");
	TextureList[2].Load("data/cut1/malygris.bmp");
	TextureList[3].Load("data/cut1/gamedev.bmp");

	LogoText = new char*[NumTextures];
	for(int i = 0; i < NumTextures; i++)
		LogoText[i] = new char[80];
	strcpy(LogoText[0], "PRODUCED BY");
	strcpy(LogoText[1], "POWERED BY");
	strcpy(LogoText[2], "ALL MUSIC BY");
	strcpy(LogoText[3], "CONTEST SPONSORED BY");

	Font = new CFont("data/fonts/trent.bmp", "data/fonts/trent.txt");
	Font->SetColor(255, 255, 255, 90);
	Font->SetSize(18);

	IntroMusic = CAudioSystem::Get()->LoadMP3("data/cut1/intro.mp3");
	CAudioSystem::Get()->PlayMP3(IntroMusic, true);

	glEnable(GL_TEXTURE_2D);
}

bool CCutScene1::Draw()
{
	Engine->SetRenderMode(CAFFEINE_ORTHO);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	CTimer::Get()->Update();

	static float SceneStart = CTimer::Get()->GetRunningTime();

	RunningTime = CTimer::Get()->GetRunningTime()-SceneStart;
	FrameTime = CTimer::Get()->GetLastFrameTime();

	if(RunningTime > SceneLength)
		return false;

	if(FlashTime > LogoTime)
		FlashTime = 0.0f;

	if(FrameTime < 1.0f && FrameTime > 0.0f)
		FlashTime += FrameTime;

	float x, y;

	x = FlashTime/LogoTime;
	y = 2*x - 1;
	y = y*y*y*y;
	y = 1-y;

	glColor3f(y, y, y);
	
	if(RunningTime < LogoTime)
		CurrentTexture = 0;
	if(RunningTime > LogoTime && RunningTime < LogoTime*2)
		CurrentTexture = 1;
	if(RunningTime > LogoTime*2 && RunningTime < LogoTime*3)
		CurrentTexture = 2;
	if(RunningTime > LogoTime*3 && RunningTime < LogoTime*4)
		CurrentTexture = 3;

	TextureList[CurrentTexture].Use();

	glBegin(GL_QUADS);
		glTexCoord2f(0, 1); glVertex3f(320-LogoWidth, 240-LogoHeight, 0);
		glTexCoord2f(0, 0); glVertex3f(320-LogoWidth, 240+LogoHeight, 0);
		glTexCoord2f(1, 0); glVertex3f(320+LogoWidth, 240+LogoHeight, 0);
		glTexCoord2f(1, 1); glVertex3f(320+LogoWidth, 240-LogoHeight, 0);
	glEnd();

	Font->DrawText(20, 20, LogoText[CurrentTexture]);

	SwapBuffers(Engine->GetContext());

	if(CInputSystem::Get()->KeyDown(VK_ESCAPE))
	{
		while(CInputSystem::Get()->KeyDown(VK_ESCAPE));
		//Unload();
		return false;
	}

	return true;
}

void CCutScene1::Unload()
{
	CAudioSystem::Get()->UnloadMP3(IntroMusic);

	delete Font;
	delete [] TextureList;
	for(int i = 0; i < NumTextures; i++)
		delete [] LogoText[i];
	delete [] LogoText;
}

