
//-------------- file start ----------------
// * caffeine engine - engine.h
// * copyright 2002-3 zealouselixir software
// * created march 16, 2002
// * engine core declaration 
// * this file contains all engine 
//   interface functions used by
//   clients of the engine, as well
//   as functions necessary for core
//   or os-specific operations.
//------------------------------------------

//------------- directives -----------------

#ifndef __CAFFEINE_ENGINE_H
#define __CAFFEINE_ENGINE_H

//---------------includes-------------------

#include "default.h"

//------------- engine files ---------------

#include "constants.h"
#include "audio.h"
#include "input.h"

#include "glcrap.h"  // much better than glext...

//------- engine capability struct ---------

struct CAFFEINE_DESC
{
	LPSTR title;
	short width; 
	short height;
	short bpp;
	RendererType render_type;
	WindowMode window_mode;
};

//---------- engine declaration ------------

class CEngine
{
//--------- interface functions ------------
	public: 
		static CEngine *Get(CAFFEINE_DESC *desc = 0) 
		{ 
			if(!Instance)
				Instance = new CEngine(desc);

			return Instance; 
		};

		HWND GetHandle() 
		{
			return WindowHandle;
		}

		HDC GetContext()
		{
			return DeviceContext;
		}

		ErrorCode Process();

//-------- constructor/destructor ----------

	protected: 
		CEngine()
		{	};

		CEngine(CAFFEINE_DESC *descriptor);

		virtual ~CEngine() 
		{
			//KillWindow();
			if(Instance)
				delete Instance;
		};

//---------- internal functions ------------
		
	private: 
		ErrorCode GetPixelFormat(UINT bpp, UINT z_buf_depth = 16, UINT stencil_depth = 0);

		ErrorCode InitOpenGL();
		ErrorCode KillOpenGL();
	public:
		ErrorCode SizeOpenGL();
		void SetRenderMode(RenderMode);
	private:

		ErrorCode CreateContexts();
		ErrorCode DeleteContexts();

		ErrorCode MakeWindow(LPSTR title, UINT width, UINT height, WindowMode mode, UINT bpp = 16);
		ErrorCode KillWindow();

		static LONG CALLBACK StaticMessageHandler(HWND handle, UINT message, WPARAM wparam, LPARAM lparam);
		LONG CALLBACK MessageHandler(HWND handle, UINT message, WPARAM wparam, LPARAM lparam);

//------------- data members ---------------

	private: 
		static CEngine *Instance;

		WNDCLASSEX WindowClass;
		LONG ExStyle;
		LONG Style;

		HWND WindowHandle;

		bool FullScreen;

		HDC DeviceContext;
		HGLRC RenderContext;
		INT PixelFormat;

		UINT BitDepth;
		UINT FrustrumDepth;

		float Far;  // frustrum dimensions
		float Near;
		float Aspect;
		float Top;
		float Bottom;
		float Left;
		float Right;

		MSG Message;
};

//------------- directives -----------------

#endif // __CAFFEINE_ENGINE_H

//------------ file end -------------------