
#ifndef CAFFEINE_GAME
#define CAFFEINE_GAME

#include "default.h"
#include "constants.h"
#include "cafmath.h"

#include "node.h"
#include "object.h"
#include "player.h"

#include "timer.h"
#include "engine.h"
#include "input.h"
#include "audio.h"

#include "bitmap.h"
#include "texture.h"
#include "font.h"

#include "camera.h"

#include "terrain.h"
#include "particle.h"
#include "sky.h"
#include "level.h"

#include "bam.h"
#include "md2.h"

#include "cutscene1.h"
#include "cutscene2.h"

#include "startup.h"

typedef enum GameState
{
	GAME_MENU_INIT            = 100,
	GAME_MENU_RUNNING         = 110,
	GAME_MENU_SHUTDOWN        = 120,

	GAME_INIT                 = 200,
	GAME_BRIEFING             = 210,
	GAME_CREDITS              = 220,
	GAME_LOADING              = 230,
	GAME_RUNNING              = 240,
	GAME_SHUTDOWN             = 250,

	GAME_CUTSCENE_ONE_INIT    = 300,
	GAME_CUTSCENE_ONE_PLAY    = 310,
	GAME_CUTSCENE_ONE_CLEANUP = 320,

	GAME_CUTSCENE_TWO_INIT    = 400,
	GAME_CUTSCENE_TWO_PLAY    = 410,
	GAME_CUTSCENE_TWO_CLEANUP = 420,

	GAME_LEVEL_ONE_INIT       = 500,
	GAME_LEVEL_ONE_RUNNING    = 510,
	GAME_LEVEL_ONE_CLEANUP    = 520,

	GAME_DISPLAY_WINNER       = 900,
	GAME_DISPLAY_LOSER        = 910,
	
	/*GAME_LEVEL_TWO_INIT       = 600,  // sorry, ran out of time :)
	GAME_LEVEL_TWO_RUNNING    = 610,
	GAME_LEVEL_TWO_CLEANUP    = 620,

	GAME_LEVEL_THREE_INIT     = 700,
	GAME_LEVEL_THREE_RUNNING  = 710,
	GAME_LEVEL_THREE_CLEANUP  = 720,
	
	GAME_LEVEL_FOUR_INIT      = 800,
	GAME_LEVEL_FOUR_RUNNING   = 810,
	GAME_LEVEL_FOUR_CLEANUP   = 820,*/
};

class CGame
{
    public:
        static CGame* Get()
        {
            if(!Instance)
                Instance = new CGame;
            return Instance;
        };

        void Run();
		void Main();
		bool HandleInput();
		int RenderFrame();
		void RenderHUD();
		//void DrawGun();  // obsolete
		GameState DisplayMenu();
		void Update();

		bool DisplayVictory(bool TheyWon);

		void LoadHUD();

    protected:
        CGame() {};

        ~CGame()
        {
            if(Instance)
                delete Instance;
            Instance = 0;
        };

    private:
        static CGame *Instance;

		GameState State;

		CEngine *GameEngine;
		CInputSystem *GameInput;
		CAudioSystem *GameAudio;
		CFont *GameFont;

		CCamera GameCamera;

		CLevel LevelOne;
		/*CLevel LevelTwo;
		CLevel LevelThree;
		CLevel LevelTFour;*/

		CLevel *CurrentLevel;  // obsolete

		HSTREAM MenuMusic;

	private:
		CTexture MainMenu;
		CTexture MenuCursor;

		CTexture HUD;
		CTexture HUDMask;
		CTexture Crosshair;
		CTexture CrosshairMask;


};

#endif
