
#include "level.h"

CLevel::CLevel()
:Terrain(0), ObjectList(0), Sky(0), EnemyModel(0), EnemyList(0)
{
}

CLevel::~CLevel()
{
	Unload();
}
	
void CLevel::Unload()
{
	CAudioSystem::Get()->UnloadMP3(LevelMusic);
	if(Terrain)
		delete Terrain;
	if(Sky)
		delete Sky;
	if(EnemyModel)
		delete EnemyModel;
	if(EnemyList)
		delete [] EnemyList;
	//if(ObjectList)
	//	delete [] ObjectList;

	Terrain = 0;
	Sky = 0;
	EnemyModel = 0;
	EnemyList = 0;
	ObjectList = 0;
}

void CLevel::Load(char *LevelFile)
{
	ifstream InFile(LevelFile, ios::nocreate);
	if(InFile.fail())
		return;

	CAudioSystem *Audio = CAudioSystem::Get();  // grab the audio manger 
	LevelMusic = Audio->LoadMP3("data/level1/level1.mp3");
	Background = Audio->LoadMP3("data/level1/wind.mp3");
	Audio->SetChannelVolume(LevelMusic, 100);
	Audio->PlayMP3(LevelMusic, true);
	Audio->SetChannelVolume(Background, 40);
	Audio->PlayMP3(Background, true);

	char TerrainHeightMap[MAX_LINE_LENGTH];
	char TerrainDetailMap[MAX_LINE_LENGTH];
	char TerrainColorMap[MAX_LINE_LENGTH];
	char LiquidFile[MAX_LINE_LENGTH];
	char SkyFile[MAX_LINE_LENGTH];
	float Scale, WaterHeight;
	float SpawnX, SpawnY, SpawnZ;
	char EnemyFile[MAX_LINE_LENGTH];
	char EnemyDescFile[MAX_LINE_LENGTH];

	char EnemyModelFile[MAX_LINE_LENGTH];

	int Type;
	CVector Location;
	int HitPoints;

	InFile >> TerrainHeightMap;
	InFile >> TerrainColorMap;
	InFile >> TerrainDetailMap;
	InFile >> LiquidFile;
	InFile >> SkyFile;
	InFile >> Scale >> WaterHeight;
	InFile >> SpawnX >> SpawnY >> SpawnZ;
	InFile >> EnemyFile;
	InFile >> EnemyDescFile;
	InFile.close();

	Terrain = new CTerrain(TerrainHeightMap, TerrainColorMap, 
		                   TerrainDetailMap, Scale, WaterHeight);
	Sky = new CSky(SkyFile);
	CPlayer::Get()->Position(CVector(SpawnX, SpawnY, SpawnZ));

	InFile.open(EnemyFile, ios::nocreate);
	if(InFile.fail())
		return;
	InFile >> EnemyModelFile;
	InFile.close();

	EnemyModel = new CMD2Model;
	EnemyModel->Load(EnemyModelFile);
	EnemyModel->SetAnimation(RUN);

	InFile.open(EnemyDescFile, ios::nocreate);
	if(InFile.fail())
		return;
	InFile >> NumEnemies;
	EnemyList = new CEnemy[NumEnemies];

	for(int ene = 0; ene < NumEnemies; ene++)
	{
		InFile >> Type;
		InFile >> Location.x;
		InFile >> Location.z;
		InFile >> HitPoints;
		EnemyList[ene] = CEnemy(Location, HitPoints);
		EnemyList[ene].SetRep(EnemyModel);
		EnemyList[ene].SetTerrain(Terrain);
		EnemyList[ene].SetAnimation(RUN);
	}

	InFile.close();
}

int CLevel::Draw(CCamera *camera) // it draws, and does boatloads of other stuff...
{
	if(Terrain)
		Terrain->Draw(camera);
	if(Sky)
		Sky->Draw(camera);

	float FrameTime = CTimer::Get()->GetLastFrameTime();

	float EnemySize;
	float CenterDist;
	CVector DistVec;

	int DeadEnemies = 0;

	for(int ene = 0; ene < NumEnemies; ene++)
	{
		if(!EnemyList[ene].Alive())
			DeadEnemies++;
		
		EnemyList[ene].AI();
		EnemyList[ene].Animate(FrameTime);
		EnemyList[ene].Draw(camera);
		
		// Cheap, last-minute coldet.
		EnemySize = EnemyList[ene].GetSize();
		DistVec = EnemyList[ene].GetLocation() - 
			              CPlayer::Get()->GetLocation();
		CenterDist = DistVec.length();

		if(CenterDist < (EnemySize + CPlayer::Get()->GetSize()))
		{
			EnemyList[ene].Collide(CPlayer::Get());
		}

		// O(n^2) operations are never nominal...
		for(int ene2 = 0; ene2 < NumEnemies; ene2++)
		{
			if(ene != ene2)  // don't collide with (*this).  duh.
			{
				DistVec = EnemyList[ene].GetLocation() - 
			              EnemyList[ene2].GetLocation();
				CenterDist = DistVec.length();

				if(CenterDist < (EnemySize + EnemyList[ene2].GetSize()))
				{
					EnemyList[ene].Collide(&EnemyList[ene2]);
				}
			}
		}

		CBullet *BulletList = CPlayer::Get()->GetBulletList();
		int NumProjectiles  = CPlayer::Get()->GetBulletNum();

		if(!BulletList)    // one of very few crash deterrents.
			continue;

		for(int proj = 0; proj < NumProjectiles; proj++)
		{
			if(BulletList[proj].Alive())
			{
				DistVec = EnemyList[ene].GetLocation() - 
			              BulletList[proj].GetLocation();
				CenterDist = DistVec.length();

				if(CenterDist < (EnemySize + BulletList[proj].GetSize()))
				{
					EnemyList[ene].Collide(&BulletList[proj]);
				}
			}
		}
	}

	if(DeadEnemies == NumEnemies)
		return 1;   // you have one! ...yeah
	else if(!CPlayer::Get()->Alive())
		return 2;   // l00zer!
	else
		return 0;
}



