
#include "md2.h"

bool CMD2Model::Load(char *FileName)
{
	char TriFileName[MAX_LINE_LENGTH];
	char TextureFileName[MAX_LINE_LENGTH];
	float Scalar;  // scaling constant for universal consistency

	ifstream DataFile(FileName, ios::nocreate);
	DataFile >> TriFileName;
	DataFile >> TextureFileName;
	DataFile >> Scalar;
	DataFile.close();

	Frame *frame;

	ifstream ModelFile(TriFileName, ios::nocreate|ios::binary);
	char *FileBuffer;
	ModelFile.seekg(0, ios::end);
	int FileLength = ModelFile.tellg();
	ModelFile.seekg(0, ios::beg);
	FileBuffer = new char[FileLength];
	ModelFile.read(FileBuffer, FileLength);

	memcpy(&Info, FileBuffer, sizeof(MD2Header));

	VertexList = new CVector[Info.NumVertices*Info.NumFrames];

	for(int framenum = 0; framenum < Info.NumFrames; framenum++)
	{
		frame = (Frame *)&FileBuffer[Info.OffsetFrames + 
			                         Info.FrameSize * framenum];

		for(int comp = 0; comp < 3; comp++)
		{
			frame->Scale[comp] *= Scalar;
			frame->Translate[comp] *= Scalar;
		}

		for(int vertnum = 0; vertnum < Info.NumVertices; vertnum++)
		{
			VertexList[Info.NumVertices * framenum + vertnum] = CVector(
			frame->Scale[0] * frame->PointList[vertnum].v[0] + frame->Translate[0],
			frame->Scale[1] * frame->PointList[vertnum].v[1] + frame->Translate[1],
			frame->Scale[2] * frame->PointList[vertnum].v[2] + frame->Translate[2]);
		}
	}

	Texture.Load(TextureFileName);

	TexIndex *coord; 
	coord = (TexIndex *)&FileBuffer[Info.OffsetTexCoords];
	TexList = new TexCoord[Info.NumTexCoords];

	for(int coordnum = 0; coordnum < Info.NumTexCoords; coordnum++)
	{
		TexList[coordnum].s = coord[coordnum].s / (float)Texture.Width;
		TexList[coordnum].t = coord[coordnum].t / (float)Texture.Height;
	}

	Mesh *tri;
	tri = (Mesh *)&FileBuffer[Info.OffsetTriangles];
	TriList = new Mesh[Info.NumTriangles];

	for(int trinum = 0; trinum < Info.NumTriangles; trinum++)
	{
		TriList[trinum].MeshIndexList[0] = tri[trinum].MeshIndexList[0];
		TriList[trinum].MeshIndexList[1] = tri[trinum].MeshIndexList[1];
		TriList[trinum].MeshIndexList[2] = tri[trinum].MeshIndexList[2];

		TriList[trinum].TexIndexList[0] = tri[trinum].TexIndexList[0];
		TriList[trinum].TexIndexList[1] = tri[trinum].TexIndexList[1];
		TriList[trinum].TexIndexList[2] = tri[trinum].TexIndexList[2];
	}

	delete [] FileBuffer;
	ModelFile.close();

	return true;
}

void CMD2Model::SetAnimation(const Animation &NewAnim) 
{
	CurrentAnimation = NewAnim;
	CurrentFrame = NewAnim.StartFrame;
	NextFrame = CurrentFrame+1;
	Percent = 0.0f;
}

void CMD2Model::SetAnimation(const Animation &NewAnim, 
							 int StartFrame, int EndFrame, float Perc) 
{
	CurrentAnimation = NewAnim;
	CurrentFrame = StartFrame;
	NextFrame = EndFrame;
	Percent = Perc;
}

void CMD2Model::Draw()
{
	CVector *CurVertices = &VertexList[Info.NumVertices*CurrentFrame];
	CVector *NextVertices = &VertexList[Info.NumVertices*NextFrame];

	CVector Point1, Point2, Point3;

	glRotatef(270, 1,0,0);

	Texture.Use();

	glBegin(GL_TRIANGLES);
		for(int tri = 0; tri < Info.NumTriangles; tri++)
		{

			Point1 = CVector(
				CurVertices[TriList[tri].MeshIndexList[0]].x + Percent*
				(NextVertices[TriList[tri].MeshIndexList[0]].x - 
				CurVertices[TriList[tri].MeshIndexList[0]].x), 

				CurVertices[TriList[tri].MeshIndexList[0]].y + Percent*
				(NextVertices[TriList[tri].MeshIndexList[0]].y - 
				CurVertices[TriList[tri].MeshIndexList[0]].y), 

				CurVertices[TriList[tri].MeshIndexList[0]].z + Percent*
				(NextVertices[TriList[tri].MeshIndexList[0]].z - 
				CurVertices[TriList[tri].MeshIndexList[0]].z));

			Point2 = CVector(
				CurVertices[TriList[tri].MeshIndexList[1]].x + Percent*
				(NextVertices[TriList[tri].MeshIndexList[1]].x - 
				CurVertices[TriList[tri].MeshIndexList[1]].x), 

				CurVertices[TriList[tri].MeshIndexList[1]].y + Percent*
				(NextVertices[TriList[tri].MeshIndexList[1]].y - 
				CurVertices[TriList[tri].MeshIndexList[1]].y), 

				CurVertices[TriList[tri].MeshIndexList[1]].z + Percent*
				(NextVertices[TriList[tri].MeshIndexList[1]].z - 
				CurVertices[TriList[tri].MeshIndexList[1]].z));

			Point3 = CVector(
				CurVertices[TriList[tri].MeshIndexList[2]].x + Percent*
				(NextVertices[TriList[tri].MeshIndexList[2]].x - 
				CurVertices[TriList[tri].MeshIndexList[2]].x), 

				CurVertices[TriList[tri].MeshIndexList[2]].y + Percent*
				(NextVertices[TriList[tri].MeshIndexList[2]].y - 
				CurVertices[TriList[tri].MeshIndexList[2]].y), 

				CurVertices[TriList[tri].MeshIndexList[2]].z + Percent*
				(NextVertices[TriList[tri].MeshIndexList[2]].z - 
				CurVertices[TriList[tri].MeshIndexList[2]].z));

			glTexCoord2f(TexList[TriList[tri].TexIndexList[0]].s,
				         TexList[TriList[tri].TexIndexList[0]].t);
			glVertex3f(Point1.x, Point1.y, Point1.z);
			glTexCoord2f(TexList[TriList[tri].TexIndexList[2]].s,
				         TexList[TriList[tri].TexIndexList[2]].t);
			glVertex3f(Point3.x, Point3.y, Point3.z);
			glTexCoord2f(TexList[TriList[tri].TexIndexList[1]].s,
				         TexList[TriList[tri].TexIndexList[1]].t);
			glVertex3f(Point2.x, Point2.y, Point2.z);
		}
	glEnd();
}

void CMD2Model::Unload()
{
	if(VertexList)
		delete [] VertexList;
	VertexList = 0;

	if(TriList)
		delete [] TriList;
	TriList = 0;

	if(TexList)
		delete [] TexList;
	TexList = 0;
}