
#ifndef CAFFEINE_MD2
#define CAFFEINE_MD2

#include "default.h"
#include "vector.h"
#include "texture.h"
#include "constants.h"

typedef struct
{
	float s;
	float t;
} TexCoord;

typedef struct
{
	short s;
	short t;
} TexIndex;

typedef struct
{
	unsigned char v[3];
	unsigned char NormalIndex;  // Junk.
} FramePoint;

typedef struct
{
	float Scale[3];
	float Translate[3];
	char Name[16];
	FramePoint PointList[1];
} Frame;

typedef struct
{
	unsigned short MeshIndexList[3];
	unsigned short TexIndexList[3];
} Mesh;

typedef struct
{
	unsigned short StartFrame;
	unsigned short EndFrame;
	char Name[16];
} Animation;

const Animation STAND         = {1,40,"STAND"};
const Animation RUN           = {41,46,"RUN"};
const Animation ATTACK        = {47,54,"ATTACK"};
const Animation PAIN_ONE      = {55,58,"PAIN_ONE"};
const Animation PAIN_TWO      = {59,62,"PAIN_TWO"};
const Animation PAIN_THREE    = {63,66,"PAIN_THREE"};
const Animation JUMP          = {67,72,"JUMP"};
const Animation FLIPOFF       = {73,84,"FLIPOFF"};
const Animation SALUTE        = {85,95,"SALUTE"};
const Animation TAUNT         = {96,112,"TAUNT"};
const Animation WAVE          = {113,123,"WAVE"};
const Animation TAUNT_POINT   = {124,135,"POINT"};
const Animation CROUCH_STAND  = {136,154,"CROUCH_STAND"};
const Animation CROUCH_WALK   = {155,160,"CROUCH_WALK"};
const Animation CROUCH_ATTACK = {161,169,"CROUCH_ATTACK"};
const Animation CROUCH_PAIN   = {170,173,"CROUCH_PAIN"};
const Animation CROUCH_DEATH  = {174,178,"CROUCH_DEATH"};
const Animation DEATH_ONE     = {179,184,"DEATH_ONE"};
const Animation DEATH_TWO     = {185,190,"DEATH_TWO"};
// Unlike cats, MD2s have only three lives.
const Animation DEATH_THREE   = {191,198,"DEATH_THREE"};


typedef struct 
{
	int ID;
	int Version;
	int SkinWidth;
	int SkinHeight;
	int FrameSize;
	int NumSkins;
	int NumVertices;
	int NumTexCoords;
	int NumTriangles;	
	int NumGLCommands;
	int NumFrames;
	int OffsetSkins;
	int OffsetTexCoords;
	int OffsetTriangles;
	int OffsetFrames;
	int OffsetGLCommands;
	int OffsetEOF;
} MD2Header;

class CMD2Model
{
	public:
		CMD2Model()
			:AnimationSpeed(3.0f), VertexList(0), TriList(0), TexList(0)
		{};

		~CMD2Model()
		{
			Unload();
		};

		bool Load(char *FileName);
		void Unload();
		void Draw();
		void SetAnimation(const Animation &NewAnim);
		void SetAnimation(const Animation &NewAnim,  int StartFrame, 
			              int EndFrame, float Perc);

	private:
		MD2Header Info;

		CTexture Texture;

		CVector *VertexList;
		Mesh *TriList;
		TexCoord *TexList;

		Animation CurrentAnimation;
		long CurrentFrame;
		long NextFrame;
		float Percent; 
		float AnimationSpeed;
};

#endif
