
#ifndef CAFFEINE_PLAYER
#define CAFFEINE_PLAYER

class CLevel;

#include "default.h"
#include "cafmath.h"
#include "timer.h"
#include "node.h"
#include "object.h"
#include "vector.h"
#include "camera.h"
#include "terrain.h"
#include "particle.h"

const float PLAYER_SPEED = 3.0f;

const float STRAFE_LEFT = -PLAYER_SPEED;
const float STRAFE_RIGHT = PLAYER_SPEED;
const float MOVE_FORWARD = PLAYER_SPEED;
const float MOVE_BACKWARD = -PLAYER_SPEED;

void DrawCube(CVector Vc);

class CBullet : public CObject
{
	public:
		CBullet()
		{
			Type = TypeList[PROJECTILE_BULLET];
			IsAlive = false;
			Size = 1.0f;
		};

		~CBullet()
		{
			Trail.Destroy();	
		};

		void Create(CVector, CVector, PARTICLE_SYSTEM_DESC&);
		void Kill();

		bool Alive() { return IsAlive; };

	protected:
		virtual void OnDraw(CCamera *camera);
		virtual void OnAnimate(float Time);

	private:
		float TimeAlive;
		bool IsAlive;
		float LifeSpan;

		CParticleSystem Trail;
};

class CPlayer : public CObject
{
	public:
		static CPlayer *Get()
		{
			if(!Instance)
				Instance = new CPlayer;
			return Instance;
		}

	protected:

		CPlayer()
			: Height(6.5f), Grounded(false), HeadAngle(0.0f), HeadBob(0.0f)
		{
			Type = TypeList[ENTITY_PLAYER];

			Rotation = CVector(0,0,0);
			Location = CVector(0,0,0);

			LoadParticleSystem("data/particle/fire_one.txt", &FireDesc);

			HitPoints = 100;
			NumBullets = 99;
			Size = 3.25f;

			BulletListSize = 8;
			BulletList = new CBullet[BulletListSize];
		}

		~CPlayer()
		{
			if(Instance)
				delete [] Instance;

			if(BulletList)
				delete [] BulletList;
			BulletList = 0;
		}

	public:
		void SetTerrain(CTerrain *NewTerrain);
		void SetCamera(CCamera *NewCamera);

		void Move(float Speed);
		void Strafe(float Speed);
		void Rotate(float Pitch, float Yaw, float Roll);

		bool Underwater();
		void Fire();
		void ApplyGravity(float Delta);
		void RigCamera();
		void Position(CVector NewPosition);

		void Jump();
		float GetHeadPos();
		float GetHeadBob();
		float GetGunBob();

		int GetAmmo();

		int GetHealth();
		void Damage(float DamagePoints);

		CBullet * GetBulletList()
		{
			return BulletList;
		};

		int GetBulletNum()
		{
			return BulletListSize;
		};

		bool Alive()
		{	return (HitPoints > 0); };

		CVector GetLocation() { return Location; };

	protected:
		virtual void OnDraw(CCamera *camera);
		virtual void OnAnimate(float Time);

		int FindBulletSlot();

	private:
		static CPlayer *Instance;

		CTerrain *Terrain;
		CCamera *Camera;

		bool Grounded;

		bool Firing;

		float HeadAngle;

		float Height;
		float FinalYPos;
		float WalkSpeed;
		float RunSpeed;
		float HeadBob;

		int HitPoints;

		float RepTime;
		int NumBullets;
		int BulletListSize;
		CBullet *BulletList;

		PARTICLE_SYSTEM_DESC FireDesc;

};

#endif